from cura.CuraApplication import CuraApplication
from UM.Math.Vector import Vector
from UM.Application import Application
from UM.Scene.Iterator.DepthFirstIterator import DepthFirstIterator

from PyQt5.QtCore import Qt


def getRect():
    left = None
    front = None
    right = None
    back = None
    for node in DepthFirstIterator(Application.getInstance().getController().getScene().getRoot()):
        if node.getBoundingBoxMesh():
            if not left or node.getBoundingBox().left < left:
                left = node.getBoundingBox().left
            if not right or node.getBoundingBox().right > right:
                right = node.getBoundingBox().right
            if not front or node.getBoundingBox().front > front:
                front = node.getBoundingBox().front
            if not back or node.getBoundingBox().back < back:
                back = node.getBoundingBox().back
    if not (left and front and right and back):
        return 0
    result = max((right - left), (front - back))
    return result


def add_screenshot(img, width, height, img_type):
    result = ""
    b_image = img.scaled(width, height, Qt.KeepAspectRatio)
    img_size = b_image.size()
    result += img_type
    datasize = 0
    for i in range(img_size.height()):
        for j in range(img_size.width()):
            pixel_color = b_image.pixelColor(j, i)
            r = pixel_color.red() >> 3
            g = pixel_color.red() >> 2
            b = pixel_color.red() >> 3
            rgb = (r << 11) | (g << 5) | b
            strHex = "%x" % rgb
            if len(strHex) == 3:
                strHex = '0' + strHex[0:3]
            elif len(strHex) == 2:
                strHex = '00' + strHex[0:2]
            elif len(strHex) == 1:
                strHex = '000' + strHex[0:1]
            if strHex[2:4] != '':
                result += strHex[2:4]
                datasize += 2
            if strHex[0:2] != '':
                result += strHex[0:2]
                datasize += 2
            if datasize >= 50:
                datasize = 0
        if i != img_size.height() - 1:
            result += '\rM10086 ;'
    return result


def take_screenshot():
    app = CuraApplication.getInstance().getMainWindow()
    if app:
        m_controller = Application.getInstance().getController()
        m_camera = m_controller.getScene().getActiveCamera()
        camera_tool = m_controller.getCameraTool()
        total_x = getRect()
        if total_x > 0:
            scale = 200 / 50 * total_x
            offset = 80 / 282 * total_x
            camera_tool.setOrigin(Vector(0, offset, 0))
            m_camera.setPosition(Vector(0, 0, scale))
            m_camera.setPerspective(True)
            m_camera.lookAt(Vector(0, 100, 0))
            camera_tool.rotateCam(0, 30)
            camera_tool.rotateCam(30, 0)
            window_image = app.grabWindow()
            sidebar_width = app.minimumWidth() / 2
            now_height = app.height() - 60
            now_width = app.width()
            pw = now_width - sidebar_width - 40
            if now_height < pw:
                cut_image = window_image.copy(40, 60, now_height, now_height)
            else:
                cut_image = window_image.copy(40, now_height - pw, pw, pw)
            return cut_image
    return None